var restify = require('restify');
var npm     = require('npm');

var server = restify.createServer({
  name: 'rest-npm',
  version: '1.0.0'
});

var commands = {
  view: [ 1, 2 ],
  search: 1
};

server.use(restify.acceptParser(server.acceptable));
server.use(restify.queryParser());
server.use(restify.bodyParser());

server.get('/:command/?', function (req, res, next) {
  var params = req.query.p;

  // No params passed
  if (!params) { params = [ ]; }

  // Only one param passed
  else if (!Array.isArray(params)) { params = [ params ]; }

  // Command is not defined in the API
  if (!commands.hasOwnProperty(req.params.command)) {
    throw(new Error('Invalid command passed '+req.params.command));
  }

  var numParamsNeeded = commands[req.params.command];

  // Variable number of params
  if (
    Array.isArray(numParamsNeeded) &&
    (params.length < numParamsNeeded[0] || params.length > numParamsNeeded[1])
  ) {
    throw(new Error(
      'Invalid command params. '+
      numParamsNeeded[0]+' to '+numParamsNeeded[1]+' '+
      'params required, '+
      'received '+params.length
    ));
  }

  // Fixed number of params
  else if (
    !Array.isArray(numParamsNeeded) &&
    params.length !== numParamsNeeded
  ) {
    throw(new Error(
      'Invalid command params. '+
      numParamsNeeded+' params required, '+
      'received '+params.length
    ));
  }

  console.log('Executing command '+req.params.command+' with params '+params);

  // Execute the command
  npm.commands[req.params.command](params, function(err, data) {
    if (err) { throw(new Error('NPM error: '+err)); }
    res.send(data);
  });
});

exports.start = function(port, commandsOverride) {

  // Were custom command definitions passed?
  commands = commandsOverride ? commandsOverride : commands;

  // Load NPM with default config options
  npm.load({ }, function(err) {
    if (err) {
      console.log('Error loading NPM', err.stack);
    } else {
      server.listen(port, function() {
        console.log('%s listening at %s', server.name, server.url);
      });
    }
  });
};

